#include <allegro.h>
#include "wcolgradgen.h" /* has changed since version 1 */

/* waits until a certain key is pressed,
   returns not until it is released again.
   e.g. wait_for_key(KEY_ESC); */
void wait_for_key(int whatkey)
{
  int kbpoll = keyboard_needs_poll();

  while(!key[whatkey])
  {
    if(kbpoll) poll_keyboard();
  }
  while(key[whatkey])
  {
    if(kbpoll) poll_keyboard();
  }
}

void show_wait_message()
{
  clear_to_color(screen,makecol(255,255,255));
  line(screen,0,0,SCREEN_W-1,SCREEN_H-1,makecol(0,0,0));
  line(screen,0,SCREEN_H-1,SCREEN_W-1,0,makecol(0,0,0));
  textout_centre_ex(screen, font, "caculating on backbuffer (stand by...)",SCREEN_W / 2, SCREEN_H / 2,makecol(0,0,0), makecol(255, 255, 255));
}

void show_user_message(int a, int b)
{
  if(a) textout_centre_ex(screen, font, "hit spacebar for next example",SCREEN_W / 2, SCREEN_H / 2 -5,makecol(0,0,0), -1);
  if(b) textout_centre_ex(screen, font, "hit escape to quit",SCREEN_W / 2, SCREEN_H / 2 +5,makecol(0,0,0), -1);
}

int main()
{
  BITMAP* backb=NULL;
  int W=640; int H=480; int bpp=32; /* hint: try changing bpp to 8 for weird look */
  col_att_list3 test3;

  /* init allegro */
  if (allegro_init()!=0)
  {
    allegro_message("Could not initialize Allegro.");
    return -1;
  }
  
  /* init keyboard */
  if (install_keyboard()!=0) 
  {
    allegro_message("Keyboard failed.");
    return -2;
  }

  /* init gfx subsystem */
  set_color_depth(bpp);
  if (set_gfx_mode(GFX_AUTODETECT_WINDOWED,W,H,0,0)!=0) /* try windowed */
  {
    if (set_gfx_mode(GFX_AUTODETECT_FULLSCREEN,W,H,0,0)!=0) /* try fullscreen, if windowed failed */
    {
      allegro_message("Could not set gfx mode.");
      return -3;
    }
  }
  set_window_title("Dynamic Color Gradient Generator Test3(*fix2*) (10/Aug/2005) -Dennis Busch-");
  
  /* init backbuffer */
  backb=create_bitmap_ex(bpp,SCREEN_W,SCREEN_H);
  if (!backb)
  {
    return -4; 
  }

  /* EXAMPLE 0 */
  show_wait_message();
  init_ca_list3(&test3);
  add_to_ca_list3(&test3,CA_POINT,CA_ATTRACT,1.0,1.0,0,0, 200,200,0, 500,NULL);
  add_to_ca_list3(&test3,CA_POINT,CA_ATTRACT,1.0,1.0,639,479,0,200,0,500,NULL);
  add_to_ca_list3(&test3,CA_POINT,CA_ATTRACT,1.0,1.0,320,240,0,0,200,500,NULL);
  add_to_ca_list3(&test3,CA_GLOBAL,CA_ATTRACT,1.0,1.0,40,0,255,255,255,0,NULL); 
  add_to_ca_list3(&test3,CA_POLYNOMIAL,CA_ATTRACT,1.0,1.0,0,0,255,200,0,200,
                  make_polynomial(3,320,320,240,-1.0,0.0,0.0,0.0));
  add_to_ca_list3(&test3,CA_POLYNOMIAL,CA_ABSORB,1.0,1.0,0,0,0,255,128,100,
                  make_polynomial(3,320,320,240,-1.0,0.0,0.0,0.0));
  render_col_grad_to_bitmap3(backb,&test3); /* backb must be a valid 32bpp bitmap 640x480 in dimensions */
  blit(backb,screen,0,0,0,0,backb->w,backb->h);
  kill_ca_list3(&test3);
  show_user_message(1,1);
  while(1)
  {
    if(keyboard_needs_poll()) poll_keyboard();
    if(key[KEY_ESC]) goto END;
    if(key[KEY_SPACE]) break;
  }
  wait_for_key(KEY_SPACE);
  /* END OF EXAMPLE 0 */

  /* EXAMPLE 1 */
  show_wait_message();
  init_ca_list3(&test3); /* don't forget to kill the list before exitting or re-using */
  add_to_ca_list3(&test3,CA_HBAR,CA_ATTRACT,1.0,1.0,0,0,0,0,128,480,NULL);
  add_to_ca_list3(&test3,CA_POINT,CA_ATTRACT,1.0,1.0,320,240,255,255,255,300,NULL); 
  render_col_grad_to_bitmap3(backb,&test3);
  blit(backb,screen,0,0,0,0,backb->w,backb->h);
  kill_ca_list3(&test3);
  show_user_message(1,1);
  while(1)
  {
    if(keyboard_needs_poll()) poll_keyboard();
    if(key[KEY_ESC]) goto END;
    if(key[KEY_SPACE]) break;
  }
  wait_for_key(KEY_SPACE);
  /* END OF EXAMPLE 1 */

  /* EXAMPLE 2 same as EXAMPLE 1, except p is set to 5 on second attractor */
  show_wait_message();
  init_ca_list3(&test3); /* don't forget to kill the list before exitting or re-using */
  add_to_ca_list3(&test3,CA_HBAR,CA_ATTRACT,1.0,1.0,0,0,0,0,128,480,NULL);
  add_to_ca_list3(&test3,CA_POINT,CA_ATTRACT,5.0,1.0,320,240,255,255,255,300,NULL); 
  render_col_grad_to_bitmap3(backb,&test3);
  blit(backb,screen,0,0,0,0,backb->w,backb->h);
  kill_ca_list3(&test3);
  show_user_message(1,1);
  while(1)
  {
    if(keyboard_needs_poll()) poll_keyboard();
    if(key[KEY_ESC]) goto END;
    if(key[KEY_SPACE]) break;
  }
  wait_for_key(KEY_SPACE);
  /* END OF EXAMPLE 2 */

  /* EXAMPLE 3 same as EXAMPLE 1, except exp is set to 5 on second attractor */
  show_wait_message();
  init_ca_list3(&test3); /* don't forget to kill the list before exitting or re-using */
  add_to_ca_list3(&test3,CA_HBAR,CA_ATTRACT,1.0,1.0,0,0,0,0,128,480,NULL);
  add_to_ca_list3(&test3,CA_POINT,CA_ATTRACT,1.0,5.0,320,240,255,255,255,300,NULL); 
  render_col_grad_to_bitmap3(backb,&test3);
  blit(backb,screen,0,0,0,0,backb->w,backb->h);
  kill_ca_list3(&test3);
  show_user_message(0,1);
  while(1)
  {
    if(keyboard_needs_poll()) poll_keyboard();
    if(key[KEY_ESC]) goto END;
    if(key[KEY_SPACE]) break;
  }
  wait_for_key(KEY_SPACE);
  /* END OF EXAMPLE 3 */

  END:
  wait_for_key(KEY_ESC);

  /* de-init everything */
  destroy_bitmap(backb); backb=NULL;
  set_gfx_mode(GFX_TEXT,80,25,0,0);
  remove_keyboard();
  allegro_exit();
  return 0;
}
END_OF_MAIN()