#ifndef     __WCOL_GRADGEN_H__
#define     __WCOL_GRADGEN_H__

/* "Dynamic Color Gradient Generator (wip V3(visually fixed))"
   using "color attractors".
   Aug. 10th 2005 -Dennis Busch- "http://www.dennisbusch.de"
   "Greetings to Allegroids all over the world!" :) 
*/

/* What does it do (see main for example usage or just compile and run):
   ---------------------------------------------------------------------
   Version 3fix (Aug. 10th 2005) (version 1 and 2 stripped(==declared obsolete))
   Whereas in the unfixed version p and exp did exactly what was described,
   their behaviour lead to "visually odd" results.
   In this fixed version p is used to modify the intensity of the color.
   And exp is used to modify the "falloff" behaviour of the intensity in
   respect to the distance from the attractor.
   (Check the three examples in "main" to see the difference.)
   Changes in detail:
   -The old "effect_modify" was deleted
   -And "render_col_grad_to_bitmap3" was changed
   -in PN_DESC the coefficient type was changed to double *grrrr to the ellipse(...)*
   -make sure you place a "." before the variables pass as coefficients to "make_polynomial",
    even if you want to pass just 0 or else you'll get wrong behaviour, because of stupid
    type promotion in the ellipse *grrr* (bet your donkey that the next version
    will not use the ellipse but rather a pointer to a dynamic length array!)
   (Still the original behaviour of version1 can be achieved by setting p=1.0,exp=1.0)

   Version 3: (Aug. 6th 2005) (introducing polynomial attractors CA_POLYNOMIAL)
   Check newly defined PN_DESC starting at line 156 and the accompanied support
   functions below that.

   Version 2: (Aug. 5th 2005) (minor changes: fixed typo, prevent div by 0)
   Added support for color "absorbers"(introducing CA_MODE for future extensions),
   they work against the "attractors" in the way that they will take away color.
   The nearer a pixel lies to one of them, the greater a fraction of the color
   specified by the "absorber" will be taken away.
   Also added the possibility to play around with "p" an "exp" which will
   change the intensity(distance)=p * distance^exp of an an "attractor"

   Version 1: (Aug. 3rd 2005) (everything is preserved, version2 uses new structs and functions)
   User(==programmer, sorry no GUI available) fills a list with a
   freely choosable number of so called "color attractors"(hehe, i just invented
   that name) and then calls a function that will fill a bitmap with colors.
   Each pixels' color will be an average of all the colors used by
   the "color attractors". The nearer a pixel lies to one of the color attractors
   the more it gets mixed with that color as a fraction of the distance relative
   to that color attractor's range.
   Note: The "color attractors" do not necessarily have to lay "inside" the bitmap. */

/* Instructions:
   -------------
   Quick start: version2: "enum CA_MODE" is new and "struct CA_DESC2"

                version1:
                Read the comments to "enum CA_TYPE", "struct CA_DESC",
                quickly fly over the function declaration "add_to_ca_list",
                (Just ignore everything else.)
*/

/* Requirements:
   -------------
   -the marvellous Allegro Library ("http://alleg.sourceforge.net/")
   -some plain C programming knowledge
   -a machine capable of displaying 32bpp color modes (look for "bbp" in main to change)
    (only if you actually want to see something, of course you can just
     render to a non-screen bitmap and then save that into a file) */

/* "Feel free to experiment, modify, change, improve, rip, use, optimize,..." ;)
   "...but in any case make sure you give me credit!" */

#include <allegro.h> /* without this, NOTHING works ;) */
#include <stdarg.h> /* for variable argument functions */

/* different "color attractor" types */
enum CA_TYPE
{
  CA_GLOBAL=0, /* a color attractor with a fixed mixin percentage to every pixel */
  CA_POINT, /* a color attractor with definite location in 2D space */
  CA_HBAR, /* ignores vertical component of distance vector in computations */
  CA_VBAR, /* ignores horizontal component of distance vector in computations */
  CA_POLYNOMIAL /* v3, a color attractor based on a polynomial function 
                   in distance computations the value of that function at that
                   x position is used, so any points of the polynomial
                   that might be nearer to the current pixel will be ignored*/
};
typedef enum CA_TYPE col_att_type;

enum CA_MODE
{
  CA_ATTRACT=0, /* ADDs fractions of its' color */
  CA_ABSORB     /* SUBTRACTs fractions of its' color */
};
typedef enum CA_MODE col_att_mode;

/* ---------------  BEGIN VERSION 3 DEFS "introducing CA_POLYNOMIAL type, PN_DESC" --------------- */
/* (NOTE: type and function names are almost the same only a 3 is put at the end of each */
struct PN_DESC /* holds a full description of a polynomial */
{
  unsigned int he; /* highest exponent */
  double* co; /* will be holding he+1 coefficients in order as follows
                polynomial(x) = (co[he]*x^he + co[he-1]*x^(he-1) + ... + co[0]) */
  int unit; /* the unit in pixels */
  int x, y; /* origin x and y in pixel position, will usually be copied from CA_DESC3
               if the PN_DESC is only used inside this programme*/
  int w; /* width of the bitmap to which the polynomial will be applied 
            (will be autoset by the render...3 function) */
  int* plut; /* will be holding w number of values pol(x)==plut(x) x in [0,w-1] */
  int changed; /* set to 1, whenever the polynomial was changed since the
                  last calculation of the lookup table, else is 0 */
};
typedef struct PN_DESC polynomial;
typedef struct PN_DESC* ppolynomial;

/* BEGIN - polynomial creation, setting and deletion */

/* recreates the lookup table for a polynomial if necessary */
int make_plut(ppolynomial PN);

/* returns a pointer to a valid polynomial, or null on failure */
/* last parameters are the he+1 coefficients in same order as described above */
ppolynomial make_polynomial(unsigned int he, int unit, int x, int y, ...);

/* adjusting the polynome after createn, each will cause the lookup table to
   be recalculated on the first render */
void set_pn_co(ppolynomial PN, unsigned int which, float);
void set_pn_unit(ppolynomial PN, int unit);
void set_pn_x(ppolynomial PN,int x);
void set_pn_y(ppolynomial PN,int y);
void set_pn_w(ppolynomial PN,int w);

/* deletion of a polynome, will be used by "kill_ca_list3" if necessary, so
   inside this programme the user won't need to call this */
void kill_polynomial(ppolynomial PN);

/* END - polynomial creation, setting and deletion */

struct CA_DESC3 /* version 3 */
{
  col_att_type t; /* one of the types in CA_TYPE */ 
  col_att_mode m; /* one of the types in CA_MODE, set to CA_ATTRACT causes equal behaviour as version1 */
  float p,exp; /* in computations: effect_modifier(euclid.distance) = p * euclid.distance^exp 
                  setting p to 1.0 and exp to 1.0 causes equal behaviour as in version one */
  int x; /* global percentage of color(actually 100-that) if t==GLOBAL; x-location otherwise */
  int y; /* y-location */
  int r,g,b; /* color triplet; values other than from 0 to 255 can cause weird
                behaviour */
  int range; /* range in which the ca is active (ignored for GLOBAL) */
  ppolynomial PN; /* points to the used polynomial if t==POLYNOMIAL */
  
/* Note to self: idea for future extension ppolynomial QN, for PN/QN types of mathfuncs */

  struct CA_DESC3* pnext; /* for color attractor lists */
};
typedef struct CA_DESC3 col_att3;
typedef struct CA_DESC3* pcol_att3;

/* a "color attractor" list */
struct CA_DESC_LIST3
{
  int link_count;
  pcol_att3 plist;
};
typedef struct CA_DESC_LIST3 col_att_list3;

/* list management v3 */
int init_ca_list3(col_att_list3* pcal);
int kill_ca_list3(col_att_list3* pcal);
int add_to_ca_list3(col_att_list3* pcal, col_att_type t, col_att_mode m, float p, float exp, int x, int y, int r, int g, int b, int range, ppolynomial PN);

/* version 3 render
   the function that does all the work described at the beginning of this file,
   takes a target bitmap and a pointer to the list of "color attractors"v3 used;
   Note to user: DON'T call this on screen or videobitmaps(sloooow!) */
void render_col_grad_to_bitmap3(BITMAP *bmp, col_att_list3* pcal);
/* ---------------  END VERSION 3 DEFS --------------- */


#endif      //__WCOL_GRADGEN_H__
